<?php
/**
 * @package plugin altruja for Joomla! 2.5 and 3.x
 * @author      Altruja
 * @copyright   Copyright (C) 2014 Altruja GmbH. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

class PlgContentAltruja extends JPlugin
{

  protected
    $email = null,
    $cachedEmail = null,
    $async = null,
    $link = null,

    $document,
    $db,
    $app,
    $language
  ;

  public function __construct(&$subject, $config = array()) {
    parent::__construct($subject, $config);

    $this->db = JFactory::getDbo();
    $this->document = JFactory::getDocument();
    $this->app = JFactory::getApplication();
    $this->language = JFactory::getLanguage();

    $this->language->load('plg_content_altruja', JPATH_ADMINISTRATOR);

    $this->email = $this->params->get('email');
    
    $this->retrieve();

    if ($this->cachedEmail != $this->email) {
      try {
        $this->fetch();
      } catch (Exception $e) {
        $this->app->enqueueMessage(JText::_('PLG_CONTENT_ALTRUJA_ERROR_EMAIL'), 'error');
        $this->cachedEmail = '';
      }

      $this->store();
    }
  }

  public function onBeforeCompileHead() {
    if ($this->app->isSite()) {
      $this->display();
    }
  }

  protected function retrieve() {
    $query = $this->db->getQuery(true);
    $query->select('custom_data');
    $query->from($this->db->quoteName('#__extensions'));
    $query->where($this->db->quoteName('name')." = ".$this->db->quote("plg_content_altruja"));
    $this->db->setQuery($query);
    $data = $this->db->loadResult();
    $data = json_decode($data);
    $this->cachedEmail = null;
    $this->async = null;
    $this->link = null;
    if ($data instanceOf stdClass) {
      $this->async = $data->async;
      $this->link = $data->link;
      $this->cachedEmail = $data->cachedEmail;
    }
  }

  protected function store() {
    $data = new stdClass();
    $data->name = 'plg_content_altruja';
    $data->custom_data = json_encode(array('async' => $this->async, 'link' => $this->link, 'cachedEmail' => $this->cachedEmail));
    $this->db->updateObject('#__extensions', $data, 'name');
  }

  protected function fetch() {

    $url = 'http://www.altruja.de/api/integration/email/' . $this->email;

    $this->link = null;
    $this->async = null;
    $this->cachedEmail = $this->email;
    
    if (class_exists('JHttpFactory')) {
      $http = JHttpFactory::getHttp();
      $response = $http->get($url);
      if ($response->code !== 200) {
        throw new UnexpectedValueException('server error');
      }
      $body = $response->body;
    } else {
      require dirname(__FILE__) . '/legacy/Snoopy.class.php';
      $snoopy = new Snoopy;
      $snoopy->fetchtext($url);
      if (false === strpos($snoopy->response_code, '200 OK')) {
        throw new UnexpectedValueException('server error');
      }
      $body = $snoopy->results;
    }

    $message = json_decode($body);

    if ($message == null) {
      throw new UnexpectedValueException('parse error');
    }

    if ($message->status != 'success') {
      throw new UnexpectedValueException('invalid');
    }

    $this->link = $message->link;
    $this->async = $message->async;
  }

  protected function display() {
    $this->document->addScriptDeclaration($this->async);
  }

}

